import requests, string
from requests.utils import requote_uri
from bs4 import BeautifulSoup, NavigableString, Tag
from skmds.classes import *


# Private module functions:

def _parse_details(soup):
    """ Parse the municipality info, mail address, and office address from mds page
    :param: soup: the bs4 soup of the page
    :return: dict: the municipality details
    """
    municipality = {}
    municipality['mail_address'] = {}
    municipality['office_address'] = {}
    
    # loop through html elements in depth first order
    for el in soup.find('fieldset').recursiveChildGenerator():
        if isinstance(el, NavigableString):
            continue
        if isinstance(el, Tag) and el.has_attr('id'):
            tag_id = el.get('id').lower()[3:]
            # parse name and category
            if tag_id == 'muni':  
                words = el.text.split(' of ')
                res = string.capwords(words[-1].lower()), words[0].lower()
                municipality['name'] = res[0]
                municipality['category'] = res[1]
            # parse assessment and population
            elif tag_id == 'assessment' or tag_id == 'population':
                if el.text == '':
                    municipality[tag_id] = None
                else:
                    municipality[tag_id] = int(el.text.replace(",", '').strip()) 
            # parse rm number
            elif tag_id == 'rmnumber':  
                try:
                    municipality['rm_number'] = int(el.text)
                except:
                    municipality['rm_number'] = None
            # parse municipality contact info
            elif tag_id == 'phone' or tag_id == 'fax' or tag_id == 'email' or tag_id == 'website':  
                if el.text == '':
                    municipality[tag_id] = None
                else:
                    municipality[tag_id] = el.text
            # parse municipality mail address
            elif tag_id[:4] == 'mail':  
                municipality['mail_address'][tag_id[4:]] = el.text
            # parse office address
            elif tag_id[:6] == 'office':  
                tag = tag_id[6:]
                if tag != 'info':
                    municipality['office_address'][tag] = el.text
    
    municipality = Municipality(municipality)
    return municipality


def _parse_personnel(soup):
    """ Parse the personnel data from the table on the mds page
    :param: soup: the bs4 soup of the page
    :return: list: list of all personnel
    """
    personnel = []
    personnel_table = soup.find('table', attrs={'id': 'tblPersonnel'})
    rows = personnel_table.find_all('tr')
    for row in rows:
        cols = row.find_all('td')
        cols = [el.text.strip() for el in cols]
        person = Personnel(cols[0][:-1], cols[1], cols[2])
        personnel.append(person)
    return personnel


def _get_links(uri):
    """ Retrieve a list of results from a given mds search url
    :param: url: the search url to parse
    :return: list: a list of the links
    """
    html = requests.get(uri).text
    soup = BeautifulSoup(html, 'lxml')
    table = soup.find('table', id='tblMunis')
    results = table.find_all('a')
    return results


def _get_municipality(uri):
    """ Retrieve the data of a municipality from the mds page.
    :param: uri: the uri of the mds page of the municipality
    :return: Municipality: municipal data from the mds
    """
    url = 'http://www.mds.gov.sk.ca/apps/pub/mds/' + uri
    html = requests.get(url).text
    soup = BeautifulSoup(html, 'lxml')

    municipality = _parse_details(soup)
    municipality.personnel = _parse_personnel(soup)
    return municipality


# Public module functions:

def query(string):
    """ Query the mds for a municipality whose name contains the given string
    :param: municipality: a string to be queried
    :return: list: a list of Municipalities that match the given string
    """
    muni = string.replace("'", "''")
    uri = requote_uri(f'http://www.mds.gov.sk.ca/apps/pub/mds/pubsrchres.aspx?muni={muni}&ReturnURL=welcome.aspx')
    links = _get_links(uri)
    results = []
    for link in links:
        m = _get_municipality(link['href'])
        results.append(m)
    return results
